// Using a fixed FOV of 90° (PI / 2 radians).
const screen_center = {'x': 640, 'y': 360, 'z': 640 - Math.tan(Math.PI / 2 / 2)};
const NEAR_PLANE = 0.4;

let camera = undefined;

class Camera 
{
	constructor() 
	{
		this.world    = {'x': 0, 'y': 0, 'z': 0};
		this.rotation = {'x': 0, 'y': 0, 'z': 0};
		
		// We will use the same values of cos and sin for multiple calculations, so we cache them.
		this.cache = {'yaw': {'cos': Math.cos(0), 'sin': Math.sin(0)}};
	}
	
	set yaw(value)
	{
		this.rotation.y = value;
		
		this.cache.yaw.cos = Math.cos(value)
		this.cache.yaw.sin = Math.sin(value);
	}
	
	get yaw() { return this.rotation.y; }
	
	RenderWorld(runtime) 
	{	
		runtime.callFunction("WorldCanvas.Clear");
		
		for (const quad of quads) 
			quad.Transform();
	
		// We only clip quads that are not completely behind the camera.
		for (const quad of quads)
			if (!quad.Is_Behind_Camera && quad.Vertex_Behind_Camera)
				quad.Clip();
		
		// Sort from furthest to nearest.
		quads.sort((a, b) => (a.Z_Distance < b.Z_Distance) ? 1 : -1);
		
		
		for (const quad of quads)
			if (!quad.Is_Behind_Camera)
				quad.Render(runtime);
	}
}












