const WALL_HEIGHT = 200;
const LAYOUT_TO_WOLRD_SCALE = 10;

let quads = [];

class Level
{
	// Moves and rotates the camera according to the player marker in the level.
	static MoveCameraToPlayer(player)
	{	
		camera.world.x = player.x * LAYOUT_TO_WOLRD_SCALE;
		camera.world.z = player.y * LAYOUT_TO_WOLRD_SCALE;
		
		camera.world.y = player.elevation;
		
		camera.yaw =  Math.PI / 2 - player.angle;
	}
	
	// Generates a list of quads from the instances of 'Block' in this layout.
	static GenerateBlocks(blocks)
	{
		for (const block of blocks)
		{	
			// Each block will generate 4 faces.
			const faces = [];

			faces.push(left_face(block));
			faces.push(bottom_face(block));
			faces.push(right_face(block));
			faces.push(top_face(block));

			for (var face of faces) 
			{
				// Multiply each axis by scaling constant. From https://stackoverflow.com/a/14810722
				for (var vertex of face) 
					Object.keys(vertex).map(function(key, index) { vertex[key] *= LAYOUT_TO_WOLRD_SCALE; });

				quads.push(new Quad(...face)); 
			}
		}

		function left_face(block) {
			return [
				{'x': block.x, 'y': 0,           'z': block.y},
				{'x': block.x, 'y': WALL_HEIGHT, 'z': block.y},
				{'x': block.x, 'y': WALL_HEIGHT, 'z': block.y + block.height},
				{'x': block.x, 'y': 0,           'z': block.y + block.height}
			];
		}

		function bottom_face(block) {
			return [
				{'x': block.x,               'y': 0,           'z': block.y + block.height},
				{'x': block.x,               'y': WALL_HEIGHT, 'z': block.y + block.height},
				{'x': block.x + block.width, 'y': WALL_HEIGHT, 'z': block.y + block.height},
				{'x': block.x + block.width, 'y': 0,           'z': block.y + block.height}
			];
		}

		function right_face(block) {
			return [
				{'x': block.x + block.width,  'y': 0,           'z': block.y},
				{'x': block.x + block.width,  'y': WALL_HEIGHT, 'z': block.y},
				{'x': block.x + block.width,  'y': WALL_HEIGHT, 'z': block.y + block.height},
				{'x': block.x + block.width,  'y': 0,           'z': block.y + block.height}
			];
		}

		function top_face(block) {
				return [
					{'x': block.x,               'y': 0,           'z': block.y},
					{'x': block.x,               'y': WALL_HEIGHT, 'z': block.y},
					{'x': block.x + block.width, 'y': WALL_HEIGHT, 'z': block.y},
					{'x': block.x + block.width, 'y': 0,           'z': block.y}
				];
			}
	}
}