class Quad 
{
	constructor(p1, p2, p3, p4) 
	{	
		// Vertices are passed in a clockwise order.
		this.vertices = [
			new Vertex(p1.x, p1.y, p1.z), 
			new Vertex(p2.x, p2.y, p2.z),
			new Vertex(p3.x, p3.y, p3.z),
			new Vertex(p4.x, p4.y, p4.z)
		];
	}
	
	Transform() 
	{
		for (const vertex of this.vertices)
			vertex.TransformPosition();
	}
	
	get Vertex_Behind_Camera()
	{
		for (const vertex of this.vertices) 
			if (vertex.rotated.z <= 0)
				return true;
		
		return false;
	}
	
	Clip()
	{
		for (const vertex of this.vertices) 
		{
			if (vertex.rotated.z <= 0)
			{
				for (const pair of this.vertices)
				{
					// Since our quads are rectangles, we can divide the vertices into two pairs: bottom and top.
					if (!(pair == vertex) && pair.world.y == vertex.world.y)
					{
						const interpolation = (pair.rotated.z - NEAR_PLANE) / (pair.rotated.z - vertex.rotated.z);
						
						vertex.rotated.z = NEAR_PLANE
						vertex.rotated.x = (1 - interpolation) * pair.rotated.x + interpolation * vertex.rotated.x;
					}
				}
			}
		}
	}
	
	Render(runtime) 
	{
		for (const vertex of this.vertices) 
		{
			const {x, y} = vertex.ScreenPosition();
			runtime.callFunction("WorldCanvas.AddPolygonPoint", x, y);
		}
		
		runtime.callFunction("WorldCanvas.FillPolygon");
		runtime.callFunction("WorldCanvas.OutlinePolygon");
		runtime.callFunction("WorldCanvas.ResetPolygon");
	}
	
	get Z_Distance()
	{
		// We calculate the Z distance of a quad with the average of both Zs.
		return (this.vertices[0].rotated.z + this.vertices[3].rotated.z) / 2; 
	}
	
	get Is_Behind_Camera()
	{
		// Returns true if all the vertices are behind the camera (negative transformed Z).
		let is_behind = true;
		
		for (const vertex of this.vertices)
			if (vertex.rotated.z > 0)
				is_behind = false;
		
		return is_behind;
	}
}



























